@@ -302,10 +302,7 @@ static void grow_LT(Sentence sent)
 
 static void init_HT(Sentence sent)
 {
-	int i;
-	for (i=0; i<HT_SIZE; i++) {
-		sent->and_data.hash_table[i] = NULL;
-	}
+	memset(sent->and_data.hash_table, 0, HT_SIZE*sizeof(Label_node *));
 }
 
 static void free_HT(Sentence sent)
@@ -351,37 +348,24 @@ void initialize_conjunction_tables(Sentence sent)
 	}
 }
 
-/**
- * This hash function that takes a connector and a seed value i.
- * It only looks at the leading upper case letters of
- * the string, and the label.  This ensures that if two connectors
- * match, then they must hash to the same place.
- */
-static int and_connector_hash(Connector * c, int i)
-{
-	i = connector_hash(c, i);
-	return (i & (HT_SIZE-1));
-}
-
 /**
  * This is a hash function for disjuncts
  */
-static int and_hash_disjunct(Disjunct *d)
+static inline int and_hash_disjunct(Disjunct *d)
 {
-	int i;
+	unsigned int i;
 	Connector *e;
 	i = 0;
 	for (e = d->left ; e != NULL; e = e->next) {
-		i = and_connector_hash(e, i);
+		i += connector_hash(e);
 	}
-	i = i + (i<<1) + randtable[i & (RTSIZE-1)];
+	i += (i<<5);
 	for (e = d->right ; e != NULL; e = e->next) {
-		i = and_connector_hash(e, i);
+		i += connector_hash(e);
 	}
 	return (i & (HT_SIZE-1));
 }
 
-
 /**
  * Returns TRUE if the disjunct is appropriate to be made into fat links.
  * Check here that the connectors are from some small set.
@@ -819,9 +803,10 @@ void print_AND_statistics(Sentence sent)
  * the table data structures.  The label field and the string field
  * are filled in appropriately.  Priority is set to UP_priority.
  */
-static void connector_for_disjunct(Sentence  sent, Disjunct * d, Connector * c) {
+static void connector_for_disjunct(Sentence sent, Disjunct * d, Connector * c)
+{
 	int h;
-	Disjunct * d1=NULL;
+	Disjunct * d1 = NULL;
 	Label_node * lp;
 
 	h = and_hash_disjunct(d);
@@ -858,8 +843,8 @@ static Disjunct * build_fat_link_substitutions(Sentence sent, Disjunct *d)
 	Connector * cl, * cr, *tl, *tr, *wc, work_connector;
 	Disjunct *d1, *wd, work_disjunct, *d_list;
 	if (d==NULL) return NULL;
-	wd = & work_disjunct;
-	wc = init_connector(& work_connector);
+	wd = &work_disjunct;
+	wc = init_connector(&work_connector);
 	d_list = NULL;
 	*wd = *d;
 	tl = d->left;
@@ -965,10 +950,10 @@ Disjunct * build_COMMA_disjunct_list(Sentence sent)
 	Disjunct *d1, *d2, *d, work_disjunct, *wd;
 	Connector work_connector1, work_connector2, *c1, *c2;
 	Connector work_connector3, *c3;
-	c1 = init_connector(& work_connector1);
-	c2 = init_connector(& work_connector2);
-	c3 = init_connector(& work_connector3);
-	wd = & work_disjunct;
+	c1 = init_connector(&work_connector1);
+	c2 = init_connector(&work_connector2);
+	c3 = init_connector(&work_connector3);
+	wd = &work_disjunct;
 
 	d1 = NULL;  /* where we put the list we're building */
 
@@ -1422,8 +1407,8 @@ int is_canonical_linkage(Sentence sent)
 	Image_node * in;
 	Parse_info pi = sent->parse_info;
 
-	dummy_connector.priority = UP_priority;
 	init_connector(&dummy_connector);
+	dummy_connector.priority = UP_priority;
 
 	build_image_array(sent);
 
