@@ -35,6 +35,7 @@ struct count_context_s
 	int     islands_ok;
 	int     null_links;
 	int     table_size;
+	int     log2_table_size;
 	Table_connector ** table;
 	Resources current_resources;
 };
@@ -59,6 +60,7 @@ static void free_table(count_context_t *ctxt)
 
 static void init_table(Sentence sent)
 {
+	int shift;
 	/* A piecewise exponential function determines the size of the
 	 * hash table. Probably should make use of the actual number of
 	 * disjuncts, rather than just the number of words.
@@ -69,20 +71,17 @@ static void init_table(Sentence sent)
 
 	if (sent->length >= 10)
 	{
-		ctxt->table_size = (1<<16);
-#if 0
-		} else if (sent->length >= 10) {
-			table_size = (1 << (((6*(sent->length-10))/30) + 10)); 
-#endif
+		shift = 12 + (sent->length) / 6 ; 
 	} 
-	else if (sent->length >= 4)
-	{
-		ctxt->table_size = (1 << (((6*(sent->length-4))/6) + 4));
-	}
 	else
 	{
-		ctxt->table_size = (1 << 4);
+		shift = 12;
 	}
+
+	/* Clamp at max 4*(1<<24) == 64 MBytes */
+	if (24 < shift) shift = 24;
+	ctxt->table_size = (1 << shift);
+	ctxt->log2_table_size = shift;
 	ctxt->table = (Table_connector**) 
 		xalloc(ctxt->table_size * sizeof(Table_connector*));
 	memset(ctxt->table, 0, ctxt->table_size*sizeof(Table_connector*));
@@ -205,20 +204,6 @@ int do_match(Sentence sent, Connector *a, Connector *b, int aw, int bw)
 		return FALSE;
 }
 
-static int hash(int table_size,
-                int lw, int rw, Connector *le, Connector *re, int cost)
-{
-	int i;
-	i = 0;
-
-	i = i + (i<<1) + randtable[(lw + i) & (RTSIZE - 1)];
-	i = i + (i<<1) + randtable[(rw + i) & (RTSIZE - 1)];
-	i = i + (i<<1) + randtable[(((long) le + i) % (table_size+1)) & (RTSIZE - 1)];
-	i = i + (i<<1) + randtable[(((long) re + i) % (table_size+1)) & (RTSIZE - 1)];
-	i = i + (i<<1) + randtable[(cost+i) & (RTSIZE - 1)];
-	return i & (table_size-1);
-}
-
 /** 
  * Stores the value in the table.  Assumes it's not already there.
  */
@@ -233,7 +218,7 @@ static Table_connector * table_store(count_context_t *ctxt,
 	n = (Table_connector *) xalloc(sizeof(Table_connector));
 	n->count = count;
 	n->lw = lw; n->rw = rw; n->le = le; n->re = re; n->cost = cost;
-	h = hash(ctxt->table_size,lw, rw, le, re, cost);
+	h = pair_hash(ctxt->log2_table_size,lw, rw, le, re, cost);
 	t = ctxt->table[h];
 	n->next = t;
 	ctxt->table[h] = n;
@@ -248,7 +233,8 @@ find_table_pointer(count_context_t *ctxt,
                    int cost)
 {
 	Table_connector *t;
-	t = ctxt->table[hash(ctxt->table_size,lw, rw, le, re, cost)];
+	int h = pair_hash(ctxt->log2_table_size,lw, rw, le, re, cost);
+	t = ctxt->table[h];
 	for (; t != NULL; t = t->next) {
 		if ((t->lw == lw) && (t->rw == rw) && (t->le == le) && (t->re == re)
 			&& (t->cost == cost))  return t;
@@ -784,7 +770,6 @@ void conjunction_prune(Sentence sent, Parse_Options opts)
 	}
 
 	init_fast_matcher(sent);
-	init_table(sent);
 	ctxt->local_sent = sent->word;
 	ctxt->null_links = (opts->min_null_count > 0);
 	/*
@@ -817,7 +802,6 @@ void conjunction_prune(Sentence sent, Parse_Options opts)
 	delete_unmarked_disjuncts(sent);
 
 	free_fast_matcher(sent);
-	free_table(ctxt);
 
 	ctxt->local_sent = NULL;
 	ctxt->current_resources = NULL;
