@@ -184,7 +184,8 @@ static void process_linkage(Linkage linkage, Parse_Options opts)
 
 	if (!linkage) return;  /* Can happen in timeout mode */
 
-	if (parse_options_get_display_union(opts))
+	if (parse_options_get_use_fat_links(opts) &&
+	    parse_options_get_display_union(opts))
 	{
 		linkage_compute_union(linkage);
 		first_sublinkage = linkage_get_num_sublinkages(linkage)-1;
@@ -306,6 +307,9 @@ static int process_some_linkages(Sentence sent, Parse_Options opts)
 
 		linkage = linkage_create(i, sent, opts);
 
+		/* Currently, sat solver returns NULL when there ain't no more */
+		if (!linkage) break;
+
 		if (verbosity > 0)
 		{
 			if ((sentence_num_valid_linkages(sent) == 1) &&
@@ -445,7 +449,6 @@ static Label strip_off_label(char * input_string)
 	case PARSE_WITH_DISJUNCT_COST_GT_0:
 		input_string[0] = ' ';
 		return c;
-		break;
 	case NO_LABEL:
 	default:
 		return NO_LABEL;
@@ -468,10 +471,13 @@ static void setup_panic_parse_options(Parse_Options opts)
 
 static void print_usage(char *str) {
 	fprintf(stderr,
-			"Usage: %s [language]\n"
-			"		  [-ppoff] [-coff] [-aoff] [-batch]\n"
+			"Usage: %s [language|dictionary location]\n"
 			"		  [-<special \"!\" command>]\n"
 			"		  [--version]\n", str);
+
+	fprintf(stderr, "\nSpecial commands are:\n");
+	opts = parse_options_create();
+	issue_special_command("var", opts, NULL);
 	exit(-1);
 }
 
@@ -514,13 +520,8 @@ int main(int argc, char * argv[])
 {
 	FILE            *input_fh = stdin;
 	Dictionary      dict;
-	Sentence        sent = NULL;
 	const char     *language="en";  /* default to english, and not locale */
-	int             pp_on=TRUE;
-	int             af_on=TRUE;
-	int             cons_on=TRUE;
 	int             num_linkages, i;
-	char            *input_string;
 	Label           label = NO_LABEL;
 	const char      *codeset;
 
@@ -558,35 +559,39 @@ int main(int argc, char * argv[])
 		setlocale(LC_CTYPE, "en_US.UTF-8");
 	}
 
-	for (; i<argc; i++) {
-		if (argv[i][0] == '-') {
-			if (strcmp("--version", argv[i])==0) {
+	for (; i<argc; i++)
+	{
+		if (argv[i][0] == '-')
+		{
+			if (strcmp("--version", argv[i])==0)
+			{
 				printf("Version: %s\n", linkgrammar_get_version());
 				exit(0);
-			} else if (strcmp("-ppoff", argv[i])==0) {
-				pp_on = FALSE;
-			} else if (strcmp("-coff", argv[i])==0) {
-				cons_on = FALSE;
-			} else if (strcmp("-aoff", argv[i])==0) {
-				af_on = FALSE;
-			} else if (strcmp("-batch", argv[i])==0) {
-			} else if (strncmp("-!", argv[i],2)==0) {
-			} else {
-				print_usage(argv[0]);
 			}
-		} else {
+			/* TBD remove these in version 5.0 */
+			else if ((strcmp("-ppoff", argv[i])==0) ||
+			         (strcmp("-coff", argv[i])==0) ||
+			         (strcmp("-aoff", argv[i])==0))
+			{
+				fprintf(stderr, "%s: Warning: %s flag ignored\n", argv[0], argv[i]);
+			}
+		}
+		else
+		{
 			print_usage(argv[0]);
 		}
 	}
 
 	opts = parse_options_create();
-	if (opts == NULL) {
+	if (opts == NULL)
+	{
 		fprintf(stderr, "%s: Fatal error: unable to create parse options\n", argv[0]);
 		exit(-1);
 	}
 
 	panic_parse_opts = parse_options_create();
-	if (panic_parse_opts == NULL) {
+	if (panic_parse_opts == NULL)
+	{
 		fprintf(stderr, "%s: Fatal error: unable to create panic parse options\n", argv[0]);
 		exit(-1);
 	}
@@ -600,32 +605,40 @@ int main(int argc, char * argv[])
 	parse_options_set_min_null_count(opts, 0);
 	parse_options_set_max_null_count(opts, 0);
 
-	if(language && *language)
+	if (language && *language)
 		dict = dictionary_create_lang(language);
 	else
 		dict = dictionary_create_default_lang();
 
-	if (dict == NULL) {
+	if (dict == NULL)
+	{
 		fprintf(stderr, "%s: Fatal error: Unable to open  dictionary.\n", argv[0]);
 		exit(-1);
 	}
 
-	/* process the command line like commands */
-	for (i=1; i<argc; i++) {
-		if ((strcmp("-pp", argv[i])==0) ||
-			(strcmp("-c", argv[i])==0) ||
-			(strcmp("-a", argv[i])==0))
+	/* Process the command line like commands */
+	for (i=1; i<argc; i++)
+	{
+		/* TBD remove these in version 5.0 */
+		if ((strcmp("-pp", argv[i]) == 0) ||
+			(strcmp("-c", argv[i]) == 0) ||
+			(strcmp("-a", argv[i]) == 0) ||
+			(strcmp("-ppoff", argv[i]) == 0) ||
+			(strcmp("-coff", argv[i]) == 0) ||
+			(strcmp("-aoff", argv[i]) == 0))
 		{
 			i++;
 		}
-		else if ((argv[i][0] == '-') && (strcmp("-ppoff", argv[i])!=0) &&
-		         (argv[i][0] == '-') && (strcmp("-coff", argv[i])!=0) &&
-		         (argv[i][0] == '-') && (strcmp("-aoff", argv[i])!=0))
+		else if (argv[i][0] == '-')
 		{
+			int rc;
 			if (argv[i][1] == '!')
-				issue_special_command(argv[i]+2, opts, dict);
+				rc = issue_special_command(argv[i]+2, opts, dict);
 			else
-				issue_special_command(argv[i]+1, opts, dict);
+				rc = issue_special_command(argv[i]+1, opts, dict);
+
+			if (rc)
+				print_usage(argv[0]);
 		}
 	}
 
@@ -640,6 +653,9 @@ int main(int argc, char * argv[])
 	/* Main input loop */
 	while (1)
 	{
+		char *input_string;
+		Sentence sent = NULL;
+
 		input_string = fget_input_string(input_fh, stdout, opts);
 		check_winsize(opts);
 
@@ -651,8 +667,8 @@ int main(int argc, char * argv[])
 			continue;
 		}
 
-		if ((strcmp(input_string, "quit\n")==0) ||
-			(strcmp(input_string, "exit\n")==0)) break;
+		if ((strcmp(input_string, "!quit") == 0) ||
+		    (strcmp(input_string, "!exit") == 0)) break;
 
 		/* We have to handle the !file command inline; its too hairy
 		 * otherwise ... */
@@ -674,7 +690,7 @@ int main(int argc, char * argv[])
 		if (special_command(input_string, dict)) continue;
 		if (parse_options_get_echo_on(opts))
 		{
-			printf("%s", input_string);
+			printf("%s ", input_string);
 		}
 
 		if (parse_options_get_batch_mode(opts))
@@ -695,6 +711,7 @@ int main(int argc, char * argv[])
 					sentence_length(sent), parse_options_get_max_sentence_length(opts));
 			}
 			sentence_delete(sent);
+			sent = NULL;
 			continue;
 		}
 
@@ -705,7 +722,12 @@ int main(int argc, char * argv[])
 		parse_options_reset_resources(opts);
 
 		num_linkages = sentence_parse(sent, opts);
-		if (num_linkages < 0) continue;
+		if (num_linkages < 0)
+		{
+			sentence_delete(sent);
+			sent = NULL;
+			continue;
+		}
 
 #if 0
 		/* Try again, this time ommitting the requirement for
@@ -768,7 +790,8 @@ int main(int argc, char * argv[])
 		{
 			if (verbosity > 0) fprintf(stdout, "Timer is expired!\n");
 		}
-		if (parse_options_memory_exhausted(opts)) {
+		if (parse_options_memory_exhausted(opts))
+		{
 			if (verbosity > 0) fprintf(stdout, "Memory is exhausted!\n");
 		}
 
@@ -796,16 +819,19 @@ int main(int argc, char * argv[])
 		else
 		{
 			int c = process_some_linkages(sent, opts);
-			if (c == EOF) break;
+			if (c == EOF)
+			{
+				sentence_delete(sent);
+				sent = NULL;
+				break;
+			}
 		}
+		fflush(stdout);
 
 		sentence_delete(sent);
 		sent = NULL;
 	}
 
-	/* Free stuff, so that mem-leak detectors don't commplain. */
-	sentence_delete(sent);
-
 	if (parse_options_get_batch_mode(opts))
 	{
 		/* print_time(opts, "Total"); */
@@ -813,6 +839,7 @@ int main(int argc, char * argv[])
 				"%d error%s.\n", batch_errors, (batch_errors==1) ? "" : "s");
 	}
 
+	/* Free stuff, so that mem-leak detectors don't commplain. */
 	parse_options_delete(panic_parse_opts);
 	parse_options_delete(opts);
 	dictionary_delete(dict);
