@@ -175,6 +175,7 @@ char * linkage_print_senses(Linkage linkage)
 
 char * linkage_print_disjuncts(Linkage linkage)
 {
+	double cost, score;
 	const char * infword;
 	const char * dj;
 	char * djs;
@@ -190,14 +191,20 @@ char * linkage_print_disjuncts(Linkage linkage)
 	 * word 0. */
 	for (w=1; w<nwords; w++)
 	{
-		/* If the word is not inflected, then sent->word[w].d is NULL */
-		if (sent->word[w].d)
-			infword = sent->word[w].d->string;
-		else
-			infword = sent->word[w].string;
+		Disjunct *disj = linkage->sent->parse_info->chosen_disjuncts[w];
+		if (NULL == disj) continue;
 
-		dj = linkage_get_disjunct(linkage, w);
-		append_string(s, "%21s    %s\n", infword, dj);
+		infword = disj->string;
+
+		dj = linkage_get_disjunct_str(linkage, w);
+		cost = linkage_get_disjunct_cost(linkage, w);
+
+#if USE_CORPUS
+		score = linkage_get_disjunct_corpus_score(linkage, w);
+		append_string(s, "%21s    %5.1f %6.3f %s\n", infword, cost, score, dj);
+#else
+		append_string(s, "%21s    %5.1f  %s\n", infword, cost, dj);
+#endif
 	}
 	djs = string_copy(s);
 	string_delete(s);
