@@ -127,23 +127,8 @@ struct prune_context_s
  */
 static inline int hash_S(Connector * c)
 {
-	const char *s;
-	unsigned int i;
-
-	if (-1 != c->hash) return c->hash;
-
-	/* djb2 hash */
-	i = 5381;
-	i = ((i << 5) + i) + c->label;
-	s = c->string;
-	while(isupper((int)*s)) /* connector tables cannot contain UTF8, yet */
-	{
-		i = ((i << 5) + i) + *s;
-		s++;
-	}
-	c->hash = (i & (CONTABSZ-1));
-	c->prune_string = s;
-	return c->hash;
+	int h = connector_hash(c);
+	return (h & (CONTABSZ-1));
 }
 
 /**
@@ -161,9 +146,9 @@ int prune_match(int dist, Connector *a, Connector *b)
 
 	if (a->label != b->label) return FALSE;
 
-	hash_S(a);
-	hash_S(b);
-	if (a->hash != b->hash) return FALSE;
+	x = hash_S(a);
+	y = hash_S(b);
+	if (x != y) return FALSE;
 
 	s = a->string;
 	t = b->string;
@@ -540,9 +525,33 @@ int connector_matches_alam(Connector * a, Connector * b)
 	return (*s == '\0');
 }
 
+
+/**
+ * This hash function that takes a connector and a seed value i.
+ * It only looks at the leading upper case letters of
+ * the string, and the label.  This ensures that if two connectors
+ * match, then they must hash to the same place.
+ */
+static int conn_hash(Connector * c, int i)
+{
+	int nb;
+	const char * s;
+	s = c->string;
+
+	i = i + (i<<1) + randtable[(c->label + i) & (RTSIZE-1)];
+	nb = is_utf8_upper(s);
+	while(nb)
+	{
+		i = i + (i<<1) + randtable[(*s + i) & (RTSIZE-1)];
+		s += nb;
+		nb = is_utf8_upper(s);
+	}
+	return i;
+}
+
 static inline int pconnector_hash(disjunct_dup_table *dt, Connector * c, int i)
 {
-	i = connector_hash(c, i);
+	i = conn_hash(c, i);
 	return (i & (ct->dup_table_size-1));
 }
 
@@ -850,8 +859,6 @@ static Connector * insert_connectors(connector_table *ct, Exp * e,
 		if (e->dir == dir)
 		{
 			Connector *dummy = connector_new();
-			dummy->label = NORMAL_LABEL;
-			dummy->priority = THIN_priority;
 			dummy->string = e->u.string;
 			insert_connector(ct, dummy);
 			dummy->next = alloc_list;
@@ -1094,27 +1101,6 @@ static void power_table_delete(power_table *pt)
 	free(pt);
 }
 
-/**
- * This hash function only looks at the leading upper case letters of
- * the connector string, and the label fields.  This ensures that if two
- * strings match (formally), then they must hash to the same place.
- * The answer must be masked to the appropriate table size.
- *
- * This is exactly the same hash function used in fast-match.
- */
-static int power_hash(Connector * c)
-{
-	const char *s;
-	int i;
-	i = randtable[c->label & (RTSIZE-1)];
-	s = c->string;
-	while (isupper((int)*s)) {  /* Connector names are not yet UTF8-capable */
-		i = i + (i<<1) + randtable[((*s) + i) & (RTSIZE-1)];
-		s++;
-	}
-	return i;
-}
-
 /**
  * The disjunct d (whose left or right pointer points to c) is put
  * into the appropriate hash table
@@ -1123,7 +1109,7 @@ static void put_into_power_table(int size, C_list ** t, Connector * c, int shal)
 {
 	int h;
 	C_list * m;
-	h = power_hash(c) & (size-1);
+	h = connector_hash(c) & (size-1);
 	m = (C_list *) xalloc (sizeof(C_list));
 	m->next = t[h];
 	t[h] = m;
@@ -1295,7 +1281,7 @@ static int right_table_search(prune_context *pc, int w, Connector *c, int shallo
 
 	pt = pc->pt;
 	size = pt->r_table_size[w];
-	h = power_hash(c) & (size-1);
+	h = connector_hash(c) & (size-1);
 	for (cl = pt->r_table[w][h]; cl != NULL; cl = cl->next)
 	{
 		if (possible_connection(pc, cl->c, c, cl->shallow, shallow, w, word_c))
@@ -1318,7 +1304,7 @@ static int left_table_search(prune_context *pc, int w, Connector *c, int shallow
 
 	pt = pc->pt;
 	size = pt->l_table_size[w];
-	h = power_hash(c) & (size-1);
+	h = connector_hash(c) & (size-1);
 	for (cl = pt->l_table[w][h]; cl != NULL; cl = cl->next)
 	{
 	  if (possible_connection(pc, c, cl->c, shallow, cl->shallow, word_c, w))
@@ -1642,10 +1628,10 @@ static void cms_table_delete(multiset_table *mt)
 
 static int cms_hash(const char * s)
 {
-	int i=0;
-	while (isupper((int)*s)) /* connector names are not yet UTF8-capable */
+	unsigned int i = 5381;
+	while (isupper((int) *s)) /* connector names are not yet UTF8-capable */
 	{
-		i = i + (i<<1) + randtable[(*s + i) & (RTSIZE-1)];
+		i = ((i << 5) + i) + *s;
 		s++;
 	}
 	return (i & (CMS_SIZE-1));
