@@ -1225,42 +1225,58 @@ void SATEncoder::pp_prune() {
 }
 
 /* TODO: replace with analyze_xxx_linkage */
-bool SATEncoder::post_process_linkage(Linkage linkage) {
-  Linkage_info li = set_has_fat_down(_sent) ? 
-    analyze_fat_linkage(_sent, _opts, PP_SECOND_PASS) : 
-    analyze_thin_linkage(_sent, _opts, PP_SECOND_PASS);
-  return li.N_violations == 0;
+bool SATEncoder::post_process_linkage(Linkage linkage)
+{
+  if (parse_options_get_use_fat_links(_opts) &&
+      set_has_fat_down(_sent)) {
+    Linkage_info li = analyze_fat_linkage(_sent, _opts, PP_SECOND_PASS);
+    return li.N_violations == 0;
+  } else {
+    Linkage_info li = analyze_thin_linkage(_sent, _opts, PP_SECOND_PASS);
+    return li.N_violations == 0;
+  }
+  return 1;
 }
 
 /*--------------------------------------------------------------------------*
  *                         D E C O D I N G                                  *
  *--------------------------------------------------------------------------*/
-Linkage SATEncoder::create_linkage() {
+Linkage SATEncoder::create_linkage()
+{
   /* Using exalloc since this is external to the parser itself. */
   Linkage linkage = (Linkage) exalloc(sizeof(struct Linkage_s));
+  memset(linkage, 0, sizeof(struct Linkage_s));
 
   linkage->num_words = _sent->length;
   linkage->word = (const char **) exalloc(linkage->num_words*sizeof(char *));
-  linkage->current = 0;
-  linkage->num_sublinkages=0;
-  linkage->sublinkage = NULL;
-  linkage->unionized = FALSE;
   linkage->sent = _sent;
   linkage->opts = _opts;
   //  linkage->info = sent->link_info[k];  
 
-  Parse_info pi = _sent->parse_info = (Parse_info) xalloc(sizeof(struct Parse_info_struct));
+  if (_sent->parse_info) {
+    Parse_info pi = _sent->parse_info;
+    for (int i=0; i< MAX_LINKS; i++) {
+      free_connectors(pi->link_array[i].lc);
+      free_connectors(pi->link_array[i].rc);
+    }
+    free_parse_info(_sent->parse_info);
+    _sent->parse_info = NULL;
+  }
+  Parse_info pi = _sent->parse_info = parse_info_new(_sent->length);
   bool fat = extract_links(pi);
   
   //  compute_chosen_words(sent, linkage);
   /* TODO: this is just a simplified version of the
      compute_chosen_words. */
   for (int i = 0; i < _sent->length; i++) {
-    linkage->word[i] = _sent->word[i].string;
+    char *s = (char *) exalloc(strlen(_sent->word[i].string)+1);
+    strcpy(s, _sent->word[i].string);
+    linkage->word[i] = s;
   }
+  linkage->num_words = _sent->length;
   pi->N_words = _sent->length;
 
-  if (!fat)
+  if (!fat || !parse_options_get_use_fat_links(_opts))
     extract_thin_linkage(_sent, _opts, linkage);
   else
     extract_fat_linkage(_sent, _opts, linkage);
@@ -1269,17 +1285,8 @@ Linkage SATEncoder::create_linkage() {
   return linkage;
 }
 
-
-void display_linkage(Linkage linkage) {
-  for (int i = 0; i < linkage_get_num_sublinkages(linkage); i++) {
-    linkage_set_current_sublinkage(linkage, i);
-    char* string = linkage_print_diagram(linkage);
-    cout << string << endl;
-    free(string);
-  }
-}
-
-void SATEncoder::generate_linkage_prohibiting() {
+void SATEncoder::generate_linkage_prohibiting()
+{
   vec<Lit> clause;
   const std::vector<int>& link_variables = _variables->link_variables();
   for (std::vector<int>::const_iterator i = link_variables.begin(); i != link_variables.end(); i++) {
@@ -1293,54 +1300,31 @@ void SATEncoder::generate_linkage_prohibiting() {
   _solver->addConflictingClause(clause);
 }
 
-void SATEncoder::solve() {
-  DEBUG_print("ZLL: ." << _solver->nAssigns());
-  //  exit(1);
+Linkage SATEncoder::get_next_linkage()
+{
+  if (l_False == _solver->solve()) return NULL;
+  Linkage linkage = create_linkage();
 
-  int num_linkages = 0;
-  int num_valid_linkages = 0;
-  int num_connected_linkages = 0;
-  
-  lbool status;
-  while ((status = _solver->solve()) != l_False)
-  {
-    _solver->printStats();
-    cout << "Linkage: ." << ++num_linkages << "." << endl;
+  std::vector<int> components;
+  bool connected = connectivity_components(components);
+  if (connected) {
+    // num_connected_linkages++;
 
-    Linkage linkage = create_linkage();
-    display_linkage(linkage);
-
-    std::vector<int> components;
-    bool connected = connectivity_components(components);
-    if (connected) {
-      num_connected_linkages++;
-
-      if (post_process_linkage(linkage)) {
-	cout << "Linkage PP OK" << endl;
-	num_valid_linkages++;
-
-	cout << "press RETURN for the next linkage" << endl;
-	cin.get();
-      } else {
-	cout << "Linkage PP NOT OK" << endl;
-      }
-
-      generate_linkage_prohibiting();
+    if (post_process_linkage(linkage)) {
+      cout << "Linkage PP OK" << endl;
+      _sent->num_valid_linkages++;
     } else {
-      cout << "Linkage DISCONNECTED" << endl;
-      generate_disconnectivity_prohibiting(components);
+	   cout << "Linkage PP NOT OK" << endl;
     }
-    cout << "--------" << endl;
 
-
-    // exit(1);
-    // TODO: delete linkage
+    generate_linkage_prohibiting();
+  } else {
+    cout << "Linkage DISCONNECTED" << endl;
+    generate_disconnectivity_prohibiting(components);
   }
-  cout << num_linkages << " linkages found. " 
-       << "(" << num_connected_linkages << " connected, "
-       << "" << num_valid_linkages << " valid after post-processing)" << endl;
 
   _solver->printStats();
+  return linkage;
 }
 
 /*******************************************************************************
@@ -1507,7 +1491,8 @@ void SATEncoder::generate_linked_min_max_planarity() {
   }
 }
 
-bool SATEncoderConjunctionFreeSentences::extract_links(Parse_info pi) {
+bool SATEncoderConjunctionFreeSentences::extract_links(Parse_info pi)
+{
   int current_link = 0;
   const std::vector<int>& link_variables = _variables->link_variables();
   std::vector<int>::const_iterator i;
@@ -1525,21 +1510,12 @@ bool SATEncoderConjunctionFreeSentences::extract_links(Parse_info pi) {
 
     Connector* connector;
 
-    connector = (Connector*)xalloc(sizeof(Connector));
-    init_connector(connector);
-    connector->label = NORMAL_LABEL;
-    connector->priority = THIN_priority;
+    connector = connector_new();
     connector->string = var->left_connector;
-    connector->next = NULL;
     pi->link_array[current_link].lc = connector;
 
-
-    connector = (Connector*)xalloc(sizeof(Connector));
-    init_connector(connector);
-    connector->label = NORMAL_LABEL;
-    connector->priority = THIN_priority;
+    connector = connector_new();
     connector->string = var->right_connector;
-    connector->next = NULL;
     pi->link_array[current_link].rc = connector;
     
     current_link++;
@@ -1563,7 +1539,8 @@ static int is_andable(Sentence sent, Connector* c, char dir) {
 	return match_in_connector_set(sent, sent->dict->andable_connector_set, c, dir);
 }
 
-void SATEncoderConjunctiveSentences::handle_null_expression(int w) {
+void SATEncoderConjunctiveSentences::handle_null_expression(int w)
+{
   // Formula is unsatisfiable, but only if w is not a conjunctive
   // word. Conjunctive words can have empty tags, but then they must
   // have fat-links attached.
@@ -1580,7 +1557,8 @@ void SATEncoderConjunctiveSentences::handle_null_expression(int w) {
 }
 
 void SATEncoderConjunctiveSentences::determine_satisfaction(int w, char* name) {
-  if (!isConnectiveOrComma(w)) {
+  if (!parse_options_get_use_fat_links(_opts) ||
+      !isConnectiveOrComma(w)) {
     // Non-conjunctive words must have their tags satisfied
     generate_literal(Lit(_variables->string(name))); 
   } else {
@@ -1601,7 +1579,7 @@ void SATEncoderConjunctiveSentences::generate_satisfaction_for_connector(int wi,
   int low = (dir == '-') ? 0 : wi + 1;
   int hi  = (dir == '-') ? wi : _sent->length;
 
-  // detrmine if opposite of (wi, pi) is andable
+  // determine if opposite of (wi, pi) is andable
   Connector conn;
   init_connector(&conn);
   conn.label = NORMAL_LABEL;
@@ -1646,7 +1624,8 @@ void SATEncoderConjunctiveSentences::generate_satisfaction_for_connector(int wi,
     }
 
     // Commas cannot be directly connected if they have fat links down
-    if (isComma(_sent, wj)) {
+    if (parse_options_get_use_fat_links(_opts) &&
+        isComma(_sent, wj)) {
       vec<Lit> clause;
       char str[MAX_VARIABLE_NAME];
       sprintf(str,"fl_d_%d", wj);
@@ -2668,7 +2647,8 @@ void SATEncoderConjunctiveSentences::get_satisfied_link_top_cw_connectors(int wo
   }
 }
 
-bool SATEncoderConjunctiveSentences::extract_links(Parse_info pi) {
+bool SATEncoderConjunctiveSentences::extract_links(Parse_info pi)
+{
   int current_link = 0;
   bool fat = false;
 
@@ -2682,9 +2662,11 @@ bool SATEncoderConjunctiveSentences::extract_links(Parse_info pi) {
 
     // Check if words are connected with a fat-link
     bool fl_lr = 0 < var->left_word && var->right_word < _sent->length - 1 && 
+      parse_options_get_use_fat_links(_opts) &&
       isConnectiveOrComma(var->right_word) && 
       _solver->model[_variables->fat_link(var->left_word, var->right_word)] == l_True;
     bool fl_rl =  0 < var->left_word && var->right_word < _sent->length - 1 && 
+      parse_options_get_use_fat_links(_opts) &&
       isConnectiveOrComma(var->left_word) && 
       _solver->model[_variables->fat_link(var->right_word, var->left_word)] == l_True;
     if (fl_lr || fl_rl) {
@@ -2694,21 +2676,14 @@ bool SATEncoderConjunctiveSentences::extract_links(Parse_info pi) {
 
       Connector* connector;
 
-      connector = (Connector*)xalloc(sizeof(Connector));
-      init_connector(connector);
-      connector->label = NORMAL_LABEL;
+      connector = connector_new();
       connector->priority = fl_lr ? UP_priority : DOWN_priority;
       connector->string = "fat";
-      connector->next = NULL;
       pi->link_array[current_link].lc = connector;
 
-
-      connector = (Connector*)xalloc(sizeof(Connector));
-      init_connector(connector);
-      connector->label = NORMAL_LABEL;
+      connector = connector_new();
       connector->priority = fl_lr ? DOWN_priority : UP_priority;
       connector->string = "fat";
-      connector->next = NULL;
       pi->link_array[current_link].rc = connector;
       
       current_link++;
@@ -2733,7 +2708,6 @@ bool SATEncoderConjunctiveSentences::extract_links(Parse_info pi) {
 	strings_left.push_back(var->connector);
       }
 
-
       // String unification
       std::vector<const char*>::const_iterator li, ri;
       const char* left_string = strings_left[0];
@@ -2746,32 +2720,21 @@ bool SATEncoderConjunctiveSentences::extract_links(Parse_info pi) {
 	right_string = intersect_strings(_sent, right_string, *ri);
       }
       
-      
       pi->link_array[current_link].l = var->left_word;
       pi->link_array[current_link].r = var->right_word;
 
       Connector* connector;
 
-      connector = (Connector*)xalloc(sizeof(Connector));
-      init_connector(connector);
-      connector->label = NORMAL_LABEL;
-      connector->priority = THIN_priority;
+      connector = connector_new();
       connector->string = right_string;
-      connector->next = NULL;
       pi->link_array[current_link].lc = connector;
 
-
-      connector = (Connector*)xalloc(sizeof(Connector));
-      init_connector(connector);
-      connector->label = NORMAL_LABEL;
-      connector->priority = THIN_priority;
+      connector = connector_new();
       connector->string = left_string;
-      connector->next = NULL;
       pi->link_array[current_link].rc = connector;
       
       current_link++;
     }
-
   }
 
   pi->N_links = current_link;
@@ -2784,15 +2747,21 @@ bool SATEncoderConjunctiveSentences::extract_links(Parse_info pi) {
  ****************************************************************************/
 extern "C" int sat_parse(Sentence sent, Parse_Options  opts)
 {
+  SATEncoder* encoder = (SATEncoder*) sent->hook;
+  if (encoder) delete encoder;
+
   // Prepare for parsing - extracted for "preparation.c"
   build_deletable(sent, 0);
   build_effective_dist(sent, 0);
+  init_count(sent);
   count_set_effective_distance(sent);
 
-  bool conjunction = sentence_contains_conjunction(sent);
+  bool conjunction = FALSE;
+  if (parse_options_get_use_fat_links(opts)) {
+    conjunction = sentence_contains_conjunction(sent);
+  }
 
   // Create the encoder, encode the formula and solve it
-  SATEncoder* encoder;
   if (conjunction) {
     DEBUG_print("Conjunctive sentence\n");
     encoder = new SATEncoderConjunctiveSentences(sent, opts);
@@ -2800,9 +2769,43 @@ extern "C" int sat_parse(Sentence sent, Parse_Options  opts)
     DEBUG_print("Conjunction free sentence\n");
     encoder = new SATEncoderConjunctionFreeSentences(sent, opts);
   }
+  sent->hook = encoder;
   encoder->encode();
-  encoder->solve();
-  delete encoder;
+
+  // XXX this is wrong, we actually don't know yet ... 
+  // If we don't return some large number here, then the
+  // Command-line client will fail to print all of the possible
+  // linkages. Work around this by lying ... 
+  sent->num_valid_linkages = 222;
+  sent->num_linkages_post_processed = 222;
 
   return 0;
 }
+
+extern "C" Linkage sat_create_linkage(int k, Sentence sent, Parse_Options  opts)
+{
+  SATEncoder* encoder = (SATEncoder*) sent->hook;
+  if (!encoder) return NULL;
+
+  return encoder->get_next_linkage();
+}
+
+extern "C" void sat_sentence_delete(Sentence sent)
+{
+  SATEncoder* encoder = (SATEncoder*) sent->hook;
+  if (!encoder) return;
+  delete encoder;
+
+  // Don't do this parse-info-free stuff, if there's no encoder.
+  // That's because it will screw up the regular parser.
+  if (sent->parse_info) {
+    Parse_info pi = sent->parse_info;
+    for (int i=0; i< MAX_LINKS; i++) {
+      free_connectors(pi->link_array[i].lc);
+      free_connectors(pi->link_array[i].rc);
+    }
+    free_parse_info(sent->parse_info);
+    sent->parse_info = NULL;
+  }
+
+}
