@@ -3,6 +3,8 @@ extern "C" {
 }
 
 extern "C" int sat_encode(Sentence sent, Parse_Options  opts);
+extern "C" Linkage sat_create_linkage(int k, Sentence sent, Parse_Options  opts);
+extern "C" void sat_sentence_delete(Sentence sent);
 
 #include "word-tag.hpp"
 
@@ -19,11 +21,17 @@ public:
     build_word_tags();
   }
 
+  virtual ~SATEncoder()
+  {
+    delete _variables;
+    delete _solver;
+  }
+
   // Create the formula from the sentence
   void encode();
 
-  // Solve the formula and print all linkages
-  void solve();
+  // Solve the formula, returning the next linkage.
+  Linkage get_next_linkage();
  
 protected:
 
