@@ -340,13 +340,13 @@ void * xalloc(size_t size)
 	space_t *s = getspace();
 	s->space_in_use += size;
 	if (s->max_space_used < s->space_in_use) s->max_space_used = s->space_in_use;
+#endif /* TRACK_SPACE_USAGE */
 	if ((p == NULL) && (size != 0))
 	{
 		prt_error("Fatal Error: Ran out of space.\n");
 		abort();
 		exit(1);
 	}
-#endif /* TRACK_SPACE_USAGE */
 	return p;
 }
 
@@ -357,13 +357,13 @@ void * xrealloc(void *p, size_t oldsize, size_t newsize)
 	s->space_in_use -= oldsize;
 #endif /* TRACK_SPACE_USAGE */
 	p = realloc(p, newsize);
-#ifdef TRACK_SPACE_USAGE
 	if ((p == NULL) && (newsize != 0))
 	{
 		prt_error("Fatal Error: Ran out of space on realloc.\n");
 		abort();
 		exit(1);
 	}
+#ifdef TRACK_SPACE_USAGE
 	s->space_in_use += newsize;
 	if (s->max_space_used < s->space_in_use) s->max_space_used = s->space_in_use;
 #endif /* TRACK_SPACE_USAGE */
@@ -386,6 +386,7 @@ void * exalloc(size_t size)
 	s->external_space_in_use += size;
 	if (s->max_external_space_used < s->external_space_in_use)
 		s->max_external_space_used = s->external_space_in_use;
+#endif /* TRACK_SPACE_USAGE */
 
 	if ((p == NULL) && (size != 0))
 	{
@@ -393,7 +394,6 @@ void * exalloc(size_t size)
 		abort();
 		exit(1);
 	}
-#endif /* TRACK_SPACE_USAGE */
 	return p;
 }
 
@@ -711,35 +711,6 @@ FILE *dictopen(const char *filename, const char *how)
 	return fh;
 }
 
-/* ======================================================== */
-/*
- * A static array of precomputed random values. This is used in a
- * read-only fashion, and thus is thread-safe, despite being a global.
- * Its initialized only once, during startup.
- */
-
-unsigned int randtable[RTSIZE];
-
-/* There is a legitimate question of whether having the hash function	*/
-/* depend on a large array is a good idea.  It might not be fastest on   */
-/* a machine that depends on caching for its efficiency.  On the other   */
-/* hand, Phong Vo's hash (and probably other linear-congruential) is	 */
-/* pretty bad.  So, mine is a "competitive" hash function -- you can't   */
-/* make it perform horribly.											 */
-
-void init_randtable(void)
-{
-	int i;
-	static int rand_table_inited = FALSE;
-	if (rand_table_inited) return;
-	rand_table_inited = TRUE;
-
-	srand(10);
-	for (i=0; i<RTSIZE; i++) {
-		randtable[i] = rand();
-	}
-}
-
 /* ======================================================== */
 /* Locale routines */
 
