@@ -188,29 +188,6 @@ X_node * catenate_X_nodes(X_node *d1, X_node *d2)
 /* ======================================================== */
 /* Connector utilities ... */
 
-/**
- * This hash function that takes a connector and a seed value i.
- * It only looks at the leading upper case letters of
- * the string, and the label.  This ensures that if two connectors
- * match, then they must hash to the same place.
- */
-int connector_hash(Connector * c, int i)
-{
-	int nb;
-	const char * s;
-	s = c->string;
-
-	i = i + (i<<1) + randtable[(c->label + i) & (RTSIZE-1)];
-	nb = is_utf8_upper(s);
-	while(nb)
-	{
-		i = i + (i<<1) + randtable[(*s + i) & (RTSIZE-1)];
-		s += nb;
-		nb = is_utf8_upper(s);
-	}
-	return i;
-}
-
 /**
  * free_connectors() -- free the list of connectors pointed to by e
  * (does not free any strings)
@@ -240,7 +217,7 @@ Connector * connector_new(void)
 	Connector *c = (Connector *) xalloc(sizeof(Connector));
 	c->length_limit = UNLIMITED_LEN;
 	c->string = "";
-	c->label = 0;
+	c->label = NORMAL_LABEL;
 	c->hash = -1;
 	c->priority = THIN_priority;
 	c->multi = FALSE;
@@ -325,8 +302,15 @@ void exfree_link(Link * l)
  */
 static int connector_set_hash(Connector_set *conset, const char * s, int d)
 {
-	int i;
-	for(i=d; isupper((int)*s); s++) i = i + (i<<1) + randtable[(*s + i) & (RTSIZE-1)];
+	unsigned int i;
+	/* djb2 hash */
+	i = 5381;
+	i = ((i << 5) + i) + d;
+	while (isupper((int) *s)) /* connector tables cannot contain UTF8, yet */
+	{
+		i = ((i << 5) + i) + *s;
+		s++;
+	}
 	return (i & (conset->table_size-1));
 }
 
@@ -339,8 +323,6 @@ static void build_connector_set_from_expression(Connector_set * conset, Exp * e)
 	{
 		c = connector_new();
 		c->string = e->u.string;
-		c->label = NORMAL_LABEL;		/* so we can use match() */
-		c->priority = THIN_priority;
 		c->word = e->dir;       /* just use the word field to give the dir */
 		h = connector_set_hash(conset, c->string, c->word);
 		c->next = conset->hash_table[h];
